unit Htquerf;

{ Authors: Michael Ax and Ann Lynnworth, HREF Tools Corp.
  Copyright (c) 1995,1996 HREF Tools Corp.  All Rights Reserved.

  For updates & other news, please visit our web site at http://www.href.com/.

  Our License Agreement for Free and Trial-Basis items is posted on the web.
  Please <A HREF="http://www.href.com/pub/freelicn.html">Read It Now</A>.
  If you agree to be bound to all its terms you may use our software,
  otherwise you may not.  Expiration dates do apply to the usage of some
  files.
}

{----------------------------------------------------------------------------}

{ updated for EEP8 on 2/21/96 by Ann Lynnworth

  This project demonstrates how to do a fairly generic search application
  using WebHub.  The tablename and search capabilities are defined in
  the application-level .ini file (ht\htchunks\htdemo\htquer.ini).
  The default table to be used is the employee.db file, but you
  can hook it up to any table.
}

interface

uses
  SysUtils, Messages, Classes, Graphics, Controls, Forms, Dialogs
, Buttons, StdCtrls, Menus, ExtCtrls
{$IFDEF WIN32}
, Windows
{$ELSE}
, WinProcs, WinTypes
{$ENDIF}
, Toolbar
, TpMenu, TpMemo, WebMemo, Combobar
, WebTypes, WebVars, WebBase, WebCore, WebApp, WebCall
, Webmenu, HtmlBase, HtmlCore, HtmlSend, WebInfo, WebBrows,
  UpdateOk, WebIniFL, WebHttp, WebServ, IniLink
  ,WebForm, Restorer, TpApplic, TpLabel, WebLink, WebPage, WebPHub,
  UcString,  {for splitstring()}
  UcOther,   {for formatCurrency()}
  WebConst,  {for cCheckboxes constant}
  TpAbout, WdbLink, WdbScan, WdbGrid, WdbSorce, DBTables, DB, Grids,
  DBGrids, DBCtrls
  ;

type
  TQueryDemoForm = class(TWebAppForm)
    StatusBar1: TtpStatusBar;
    StatusPanel1: TtpStatusPanel;
    WebAppOutput: TWebAppOutput;
    WebInfo: TWebInfo;
    WebIniFileLink: TWebIniFileLink;
    WebServer: TWebServer;
    WebBrowser: TWebBrowser;
    WebCommandLine1: TWebCommandLine;
    WebAppHTQR: TWebApp;
    WebSession0: TWebSession;
    tpComponentPanel1: TtpComponentPanel;
    tpAppRole1: TtpAppRole;
    FormRestorer1: TFormRestorer;
    DoMenuBarFile: TtpDfmMenuItem;
    DoExit: TtpDfmMenuItem;
    DoMenuBarHelp: TtpDfmMenuItem;
    DoContents: TtpDfmMenuItem;
    DoTopicSearch: TtpDfmMenuItem;
    DoHowtouseHelp: TtpDfmMenuItem;
    DoTtpDfmMenuItem: TtpDfmMenuItem;
    DoAbout: TtpDfmMenuItem;
    DoActionComponents: TtpMenuItem;
    DoWebPages: TtpMenuItem;
    WebMenu1: TWebMenu;
    tpComboBar1: TtpComboBar;
    ToolBar2: TtpToolBar;
    DoMenuItemView: TtpMenuItem;
    DoMenuItemTool: TtpMenuItem;
    DoMenuItemVerb: TtpMenuItem;
    DoMenuItemIdle: TtpMenuItem;
    tpToolButtonBrowser: TtpToolButton;
    tpToolButtonMemo: TtpToolButton;
    tpToolButtonActivated: TtpToolButton;
    tbBlankPage: TtpToolButton;
    tbUserPage: TtpToolButton;
    tbMemoPage: TtpToolButton;
    Notebook: TNotebook;
    WebHtmlMemo1: TWebHtmlMemo;
    tpLabel1: TtpLabel;
    tpLabel2: TtpLabel;
    HTQR_ANSWER: TWebPage;
    DataSource1: TDataSource;
    Table1: TTable;
    BatchMove1: TBatchMove;
    Query1: TQuery;
    TableComplete: TTable;
    WebDataSource1: TWebDataSource;
    grid: TWebDataGrid;
    tPackAboutBox1: TtPackAboutBox;
    HTQR_DETAIL: TWebPage;
    Label1: TLabel;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    procedure SelectPage(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure HTQR_ANSWERSection(Sender: TObject; Section: Integer;
      var Chunk, Options: OpenString);
    procedure Query1BeforeOpen(DataSet: TDataset);
    procedure Query1AfterOpen(DataSet: TDataset);
    procedure gridHotField(Sender: TWebDataScan; aField: TField;
      var s: OpenString);
    procedure HTQR_DETAILSection(Sender: TObject; Section: Integer;
      var Chunk, Options: OpenString);
    procedure WebAppHTQREventMacro(Sender: TWebOutputApp; const aMacro,
      aParams, aID: String);
    procedure DoAboutClick(Sender: TObject);
  private
    { Private declarations }
    procedure loadCustomSettings;
    function  getNextDBFilename:String;
  public
    { Public declarations }
  end;

var
  QueryDemoForm: TQueryDemoForm;

implementation

{$R *.DFM}

{----------------------------------------------------------------------------}

var
  queryInfo: TStringList;   { handy place to store SQL facts; This
                              could be a property of TMyApp -- see
                              OrderTotal in the Fish Store. }
  nextTempNum:integer;      { used to cycle thru temp table names }
  initFlag: Boolean;        { flag for use with temporary workaround }

const
  maxSimulQueries=25;

{---------------------------------------------------------------}

procedure TQueryDemoForm.FormCreate(Sender: TObject);
begin
  SelectPage(tbBlankPage);
  QueryInfo:=TStringList.create;
  initFlag:=false;
  nextTempNum:=0;
end;

{---------------------------------------------------------------}

procedure TQueryDemoForm.SelectPage(Sender: TObject);
begin
  with TtpToolButton(Sender) do begin
    NoteBook.PageIndex:=Tag;
    Down:=True;
    end;
end;

procedure TQueryDemoForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  QueryInfo.free;
end;

procedure TQueryDemoForm.FormActivate(Sender: TObject);
begin
  if initFlag then exit;
  loadCustomSettings;  { build QueryInfo list }
  initFlag:=true;
end;

{----------------------------------------------------------------------------}

procedure TQueryDemoForm.loadCustomSettings;
begin
  with WebIniFileLink do begin
    IniFilename:=WebAppHTQR.inifilename;
    { read in an entire section, ignoring ; comment lines }
    ReadNoComment( 'TWebApp.QuerySpec', QueryInfo );
    end;
  { use one fact now; set the database path }
  Query1.Databasename:=QueryInfo.Values['TablePath'];
  Table1.Databasename:='PRIV'; {temp table}
  TableComplete.Databasename:=QueryInfo.Values['TablePath'];
  TableComplete.Tablename:=queryInfo.Values['Tablename'];
  TableComplete.open;
end;

{----------------------------------------------------------------------------}

procedure TQueryDemoForm.HTQR_ANSWERSection(Sender: TObject;
  Section: Integer; var Chunk, Options: OpenString);
begin
  if section>1 then exit;
  if WebAppHTQR.Command=cCheckboxes then begin
    {Reset the query when we arrive on this page, coming from homepage}
    {Leave it alone if command is something like grid.next }
    Query1.close;
    Query1.open;
    end;
end;

procedure TQueryDemoForm.Query1BeforeOpen(DataSet: TDataset);
var
  a1,a2:string[128];
begin
 {Here's what we're trying to come up with (you can figure out
  this syntax using Database Desktop, File|New|Query, Query|ShowSQL ...
  SELECT DISTINCT EmpNo, LastName, FirstName, PhoneExt
  FROM "EMPLOYEE.DB" WHERE (LastName LIKE 'B%')  }

  with TQuery( DataSet ), queryInfo do begin
    sql.clear;
    sql.add( 'SELECT ' + Values['PrimaryKeyField'] + ', ' );
    sql.add( Values['DisplayField2'] + ', ' );
    sql.add( Values['DisplayField3'] + ', ' );
    {4th field is done differently to show alternate coding technique}
    SplitString( Values['DisplayField4'], ',', a1, a2 );
    sql.add( a1 + ' ' );
    sql.add( 'FROM ' + queryInfo.Values['Tablename'] + ' ' );
    sql.add( 'WHERE ' + queryInfo.Values['SearchField'] + ' LIKE ' );
    sql.add( '''' + WebAppHTQR.Literal['FindMe']+ '%''' );
    end;
end;

function TQueryDemoForm.getNextDBFilename:String;
begin
  inc(NextTempNum);
  if NextTempNum>maxSimulQueries then NextTempNum:=1;
  result:='qry'+intToStr(NextTempNum)+'.db';
end;

procedure TQueryDemoForm.Query1AfterOpen(DataSet: TDataset);
var
  a1,a2:string[128];
begin

  with BatchMove1 do begin
    Source:=Query1;
    table1.close;
    table1.tableName:=getNextDBFilename;
    Destination:=table1;
    Mode:=batCopy;
    execute;
    query1.close;
    table1.open;
    end;
  with Table1, queryInfo do begin
    {Generally, using a hardcoded numeric index into fields[] is not good.}
    {It will work in this case.}
    fields[0].displayLabel:=Values['Label1'];
    fields[1].displayLabel:=Values['Label2'];
    fields[2].displayLabel:=Values['Label3'];
    {Here's an alternate approach for the 4th field.}
    SplitString( Values['DisplayField4'], ',', a1, a2 );
    fieldByName(a1).displayLabel:=a2;
    {set hot field}
    fieldByName(Values['HotField']).tag:=HotFieldTag;
    end;
end;

{----------------------------------------------------------------------------}

procedure TQueryDemoForm.gridHotField(Sender: TWebDataScan; aField: TField;
  var s: OpenString);
begin
  {This will only work if HTML tables are turned on.  not for preformat=true }
  {See the Fish Store for additional tips.}
  {JUMP is a built-in WebHub macro. Syntax is: JUMP|pageId,command|visiblePhrase }
  s:='<TD>' +
     '%=JUMP|detail,'+
     aField.DataSet.fields[0].asString+  {command (4th param on URL)}
     '|'+aField.asString+'=%';
end;

procedure TQueryDemoForm.HTQR_DETAILSection(Sender: TObject;
  Section: Integer; var Chunk, Options: OpenString);
var
  s:string[128];
begin
  if section>1 then exit;
  {our job here is just to locate to the correct record }
  s:=WebAppHTQR.Command;
  with TableComplete do begin
    case fields[0].DataType of
      ftString: findKey( [s] );
      ftInteger: findKey( [strToInt(s)] );
      else ; {!!! should do more datatypes }
      end;
    end;
end;

{----------------------------------------------------------------------------}



procedure TQueryDemoForm.WebAppHTQREventMacro(Sender: TWebOutputApp;
  const aMacro, aParams, aID: String);
var
  fld:TField;
  lMacro:String;
  s:String;
begin
  lMacro:=lowercase(aMacro);
  { create the meaning of the getfield macro }
  { usage: getfield,fieldname -- see mypages.htm }
  with WebAppHTQR.WebOutput do begin
    if lMacro = 'getfield' then begin
      with TableComplete do begin
        fld:=findField(aParams);  { aParams is info after comma, i.e. fieldname }
        if fld<>nil then begin
          s:=fld.asString;
          if fld.DataType=ftCurrency then
            {This doesn't help salary which is declared as N not $ datatype}
            {But maybe you'll use it another time.}
            s:=FormatCurrency(strToFloat(s));
          if fld.DataType=ftMemo then
            with fld as TMemoField do s := Text;
          Send(s);
          end
        else
          SendComment( 'Invalid fieldname: ' + aParams );
        end;
      end
    else begin
      SendComment( 'Invalid custom macro event: ' + aMacro );
      end;
    end;
end;

procedure TQueryDemoForm.DoAboutClick(Sender: TObject);
begin
   TPackAboutBox1.execute;
end;

end.
